      program nearby

c  Read database of frequencies and links of 18 GHz Fixed Service (FS)
c  and examine feasibility of merging links of higher and lower bands
c  by identifying receiving terminals that are close to transmitting
C  antennas.

      parameter(
     &   MAXLINKS = 131072,
     &   CLOSE = 0.1)			!km
      character*32 input_file /'records'/
      character*84 record
      character*80 output_line, csv_line
      character*32 output_file /'too_close.csv'/
      character*2 type_of_conflict
      character*12 txcall, call_sign(MAXLINKS)
      character*6 tx, rx, txsite(MAXLINKS), rxsite(MAXLINKS), terminals(2)
      character*12 band_limits(5)/
     &   ' 12700-13300',
     &   ' 17700-17740',
     &   ' 17740-18140',
     &   ' 18140-18300',
     &   ' 18300-18820'/
      double precision seconds(4)
      double precision tlatsecs(MAXLINKS), tlonsecs(MAXLINKS)
      double precision rlatsecs(MAXLINKS), rlonsecs(MAXLINKS)
      common /sites/ tlatsecs, tlonsecs, rlatsecs, rlonsecs
      double precision freq, frequency(MAXLINKS)
      integer band(MAXLINKS), pass
      integer num_conflicts(2), total_conflicts
      integer grand_total(2) /2*0/
      integer num_precluded(2) /2*0/
      integer num_hits(2, 5, 5) /50*0/
      integer inband(5) /5*0/
      integer matrix(3, 2)
      integer out
      logical uprooted, protected, neighboring, first, ignore

c  Statement functions

      uprooted(link) =  (band(link) .eq. 2) .or. (band(link) .eq. 5)
      protected(pass, link) = (pass .eq. 1 .and. band(link) .eq. 1)
     &  .or. (pass .eq. 2 .and. ((band(link) .eq. 3 .or. band(link) .eq. 4)))
      ignore(link) = mod(frequency(link), 1.0) .eq. 0.25 .and. band(link) .eq. 4

c  Read frequency and site coordinates of links

      call getlun(in)
      open(in, file=input_file, status='old')
      numrec = 0
      num_uprooted = 0
      num_protected = 0
  100 continue
      read(in, '(a)', end=104) record
      read(record, 1000, err=102) freq, txcall,
     &   (seconds(i), i=1, 4), tx, rx
      numrec = numrec + 1
      if (numrec .gt. MAXLINKS) stop
     &   'Too many records.  Recompile with adjusted parameters'
      tlatsecs(numrec) = seconds(1)
      tlonsecs(numrec) = seconds(2)
      rlatsecs(numrec) = seconds(3)
      rlonsecs(numrec) = seconds(4)
      txsite(numrec) = tx
      rxsite(numrec) = rx
      call bandplan(freq, iband)		!get band from frequency
      if (iband .gt. 5) then
         write(0, '(a,i6)')
     &      'Frequency out of range in record number', numrec
         stop 'Bad record'
      end if
      frequency(numrec) = freq
      call_sign(numrec) = txcall
      band(numrec) = iband
      inband(iband) = inband(iband) + 1
      if (uprooted(numrec)) num_uprooted = num_uprooted + 1
      if (protected(1, numrec) .or. protected(2, numrec))
     &   num_protected = num_protected + 1
      goto 100
  102 continue
      write(0, '(a,i,a)') 'Error reading record', numrec+1,
     &   ' of input file'
      stop 'Fix input file at indicated record number'
  104 continue
      write(0, '(3a)') 'input file is "',
     &   input_file(1:lnblnk(input_file)), '"'
      write(0, '(a,f5.1,a)') 'Too close = ', CLOSE*1e3, ' meters'
      write(0, '(a,i2,a,i6,a)')
     &   ('Band', i, band_limits(i), inband(i), ' records', i=1, 5)
      write(0, '(a,t19,i6,a)') 'Total', numrec, ' records'
      write(0, '(i6,a/i6,a)') num_uprooted, ' links uprooted  (bands 2, 5)',
     &   num_protected, ' protected links (bands 1, 3, 4)'
      close(in)

c  Identify uprooted licensees that have protected sites nearby

      call getlun(out)
      open(out, file=output_file, status='new')
      write(*, 1010)
      do link2 = 1, numrec
         if (uprooted(link2)) then
            do pass = 1, 2
               first = .true.
               link1 = 1
               do while (link1 .lt. numrec)
                  if (protected(pass, link1) .and. .not. ignore(link1)) then
                     call neighbors(CLOSE, link1, link2,
     &                  neighboring, type_of_conflict, distance)

                     if (first .and. neighboring) then
                        first = .false.
                        num_conflicts(pass) = 1
                        call set_link_dirs(type_of_conflict,
     &                     link1_dir, link2_dir)
                        call set_site_names(link1_dir, link2_dir,
     &                      txsite(link1), rxsite(link1),
     &                      txsite(link2), rxsite(link2), terminals)
                        write(output_line, 1012)
     &                     call_sign(link1), frequency(link1),
     &                     type_of_conflict(1:1),
     &                     call_sign(link2), frequency(link2),
     &                     type_of_conflict(2:2),
     &                     distance*1e3, terminals
                        write(csv_line, 1013)
     &                     call_sign(link1), frequency(link1),
     &                     type_of_conflict(1:1),
     &                     call_sign(link2), frequency(link2),
     &                     type_of_conflict(2:2),
     &                     distance*1e3, terminals
                        num_hits(pass, band(link1), band(link2)) = 
     &                     num_hits(pass, band(link1), band(link2)) + 1
                        num_precluded(pass) = num_precluded(pass) + 1
                     end if
                     if (neighboring) num_conflicts(pass) =
     &                  num_conflicts(pass) + 1

                  end if
                  link1 = link1 + 1
               end do
               if (.not. first) then
                  write(*, '(a,i6)') output_line, num_conflicts(pass)
                  write(out, '(a62,a1,i4)') csv_line, ',', num_conflicts(pass)
               end if
            end do
            total_conflicts = num_conflicts(1) + num_conflicts(2)
            grand_total(1) = grand_total(1) + num_conflicts(1)
            grand_total(2) = grand_total(2) + num_conflicts(2)
         end if
      end do
      call flush(6)

      matrix(1, 1) = num_hits(1, 1, 2)
      matrix(1, 2) = num_hits(1, 1, 5)
      matrix(2, 1) = num_hits(2, 3, 2)
      matrix(2, 2) = num_hits(2, 3, 5)
      matrix(3, 1) = num_hits(2, 4, 2)
      matrix(3, 2) = num_hits(2, 4, 5)

      write(0, 1020) ((matrix(i, j), j=1,2), i=1,3)
      do pass = 1, 2
         write(0, '(i8,a,i2)') num_precluded(pass),  ' precluded in pass', pass
      end do

 1020 format(
     &  '                    Number of Preclusions'/
     &  '                 ------------------------------'/
     &  '                 Band No Longer     Band Being'/
     &  '                    Useful            Cleared'/
     &  'Protected Band    17700-17740       18300-18829'/
     &  '--------------   -------------      -----------'/
     &  ' 12700-13300     ', 3x, i5, 12x, i7/
     &  ' 17740-18140     ', 3x, i5, 12x, i7/
     &  ' 18140-18300     ', 3x, i5, 12x, i7)

c                     Number of Preclusions
c                  ------------------------------
c                  Band No Longer     Band Being
c                     Useful            Cleared
c Protected Band    17700-17740       18300-18829
c --------------   -------------      -----------
c  12700-13300            0                468
c  17740-18140            6               1285
c  18140-18300            0               4765

      write(0, '(i8,a)') grand_total(1), ' conflicts found in pass 1'
      write(0, '(i8,a)') grand_total(2), ' conflicts found in pass 2'
      write(0, '(i8,a)') grand_total(1) + grand_total(2), ' total conflicts'

 1000 format(f10, t11, a, t23, f12, t35, f12, t47, f12, t59, f12, t71, a, t79, a)
c17760     KGV68       00123191.0  -0425846.2  00122830.0  -0425696.2   726RT   877RT
c123456789012345678901234567890123456789012345678901234567890123456789012345678901234

 1010 format(
     & '  Terminal (T/R) of      Terminal (T/R) of      Separation'/
     & '  Protected Link         Displaced Link         between'/
     & '=====================   =====================   Protected and'/
     & '        Frequency               Frequency       Displaced Sites'/
     & 'TX CALL   (GHz)   T/R   TX CALL   (GHz)   T/R     (meters)'/
     & '------- --------- ---   ------- --------- ---   ---------------')
 1012 format(
     &   a7, 1x, f9.2, 2x, a1, 4x, a7, 1x, f9.2, 2x, a1, 4x, f8.2, 3x, 4x, a, 4x, a)
 1013 format(a7, ',', f9.2, ',', a1, ',', a7, ',', f9.2, ',', a1, ',', f8.2, ',', a, ',', a)
c  KC24154, 12787.50,T,WNER994, 17720.00,R,   38.12,  50TR,  52R ,   6

c    Terminal (T/R) of      Terminal (T/R) of      Separation
c    Protected Link         Displaced Link         between
c  =====================   =====================   Protected and
c          Frequency               Frequency       Displaced Sites
c  TX CALL   (GHz)   T/R   TX CALL   (GHz)   T/R     (meters)
c  ------- --------- ---   ------- --------- ---   ---------------
c  KC24154  12787.50  T    WNER994  17720.00  R       38.12         50TR      52R  6
c  KC24154  12787.50  T    WNER994  17720.00  R       38.12         50TR      52R         6
      end

      subroutine bandplan(freq, iband)
      double precision freq, f0, f1, f2, f3, f4

c  Frequency Range     Band
c      (MHz)           I.D.    Disposition of Incumbents
c  ---------------     ----    -------------------------
c  12700 to 13300       1      protect
c  17700 to 17740       2      move up to band 2 or 3
c  17740 to 18140       3      protect
c  18140 to 18300       4      protect
c  18300 to 18820       5      move down to 2 or 3

      data f0, f1, f2, f3, f4, f5, f6 /
     &   12700.0,		!f0
     &   13300.0,		!f1
     &   17700.0,		!f2
     &   17740.0,		!f3
     &   18140.0,		!f4
     &   18299.25,	!f5
     &   18820.0/		!f6
      if (freq .lt. f0) then
         iband = 0
      else if (freq .lt. f1) then		!between f0 and f1
         iband = 1
      else if (freq .ge. f2) then
         if (freq .lt. f3) then			!between f2 and f3
            iband = 2
         else if (freq .lt. f4) then	!between f3 and f4
            iband = 3
         else if (freq .lt. f5) then	!between f4 and f5
            iband = 4
         else if (freq .lt. f6) then	!between f5 and f6
            iband = 5
         else									!higher than f6
            iband = 6
         end if
      end if
      return
      end

      subroutine set_link_dirs(type_of_conflict, link1_dir, link2_dir)
      character*2 type_of_conflict
      integer direction(2)
      do i=1, 2
         if (type_of_conflict(i:i) .eq. 'T') then
            direction(i) = 1	!forward
         else
            direction(i) = 2	!reverse
         end if
      end do
      link1_dir = direction(1)
      link2_dir = direction(2)
      return
      end
 
      subroutine set_site_names(link1_dir, link2_dir, txsite1, rxsite1, txsite2, rxsite2,
     &   terminals)
      character*6 txsite1, rxsite1, txsite2, rxsite2, terminals(2)
      if (link1_dir .eq. 1) then
         terminals(1) = txsite1
      else
         terminals(1) = rxsite1
      end if
      if (link2_dir .eq. 1) then
         terminals(2) = txsite2
      else
         terminals(2) = rxsite2
      end if
      return
      end

      subroutine neighbors(close, link1, link2,
     &   neighboring, type_of_conflict, distance)

c  Find whether terminals of link1 and link2 are too close.  If so,
c  return the respective types (transmit or receive) of the terminals
c  and the distance from the link1 terminal to the link2 terminal.

      parameter(MAXLINKS = 131072)
      character*2 type_of_conflict		!TT, TR, RT or RR
      logical neighboring
      double precision tlatsecs(MAXLINKS), tlonsecs(MAXLINKS)
      double precision rlatsecs(MAXLINKS), rlonsecs(MAXLINKS)
      common /sites/ tlatsecs, tlonsecs, rlatsecs, rlonsecs

      distance_1t2r = dist(tlatsecs(link1), tlonsecs(link1),
     &   rlatsecs(link2), rlonsecs(link2))
      distance_1r2t = dist(rlatsecs(link1), rlonsecs(link1),
     &   tlatsecs(link2), tlonsecs(link2))

      neighboring = .false.
      distance = -1
      type_of_conflict = ' '

c  Is link2 receive site too close to link1 transmit site?

      if (distance_1t2r .lt. close) then
         neighboring =.true.
         type_of_conflict = 'TR'
         distance = distance_1t2r
         return

c  Is link1 receive site too close to link2 transmit site?

      else if (distance_1r2t .lt. close) then
         neighboring =.true.
         type_of_conflict = 'RT'
         distance = distance_1r2t
         return
      end if

      return
      end
