      subroutine views(far, beamwidth, link1, link2,
     &   mutually_visible, type_of_conflict, bearing, distance)

c  Find whether terminals of link1 and link2 are mutually visible.
c  If so, return the respective types (transmit or receive) of the
c  terminals facing each other, the distance and the bearing from
c  the link1 terminal to the link2 terminal.

      integer ACROSS, OPPOSITE, FORWARD, REVERSE
      parameter(
     &   MAXLINKS = 131072,
     &   REQUIRED_MARGIN = 60.0,		!dB
     &   FORWARD = 1, REVERSE = 2,
     &   ACROSS = 1, OPPOSITE = 2)
      character*2 type_of_conflict		!TT, TR, RT or RR
      double precision tlatsecs(MAXLINKS), tlonsecs(MAXLINKS)
      double precision rlatsecs(MAXLINKS), rlonsecs(MAXLINKS)
      integer band(MAXLINKS)
      logical mutually_visible, within_view
      real relative_bearing(2), path_len(2)	!inputs to D/U calculation
      common /bands/ band
      common /paths/ path_length(MAXLINKS), directions(2, MAXLINKS)
      common /sites/ tlatsecs, tlonsecs, rlatsecs, rlonsecs

      rev_bear(bearing) = mod(bearing+180., 360.)

      bearing_1t2r = bear(tlatsecs(link1), tlonsecs(link1),
     &   rlatsecs(link2), rlonsecs(link2))
      bearing_1r2t = bear(rlatsecs(link1), rlonsecs(link1),
     &   tlatsecs(link2), tlonsecs(link2))

      distance_1t2r = dist(tlatsecs(link1), tlonsecs(link1),
     &   rlatsecs(link2), rlonsecs(link2))
      distance_1r2t = dist(rlatsecs(link1), rlonsecs(link1),
     &   tlatsecs(link2), tlonsecs(link2))

      allow = beamwidth/2
      new_band = min(2, band(link1))
      path_len(OPPOSITE) = path_length(link1)	!length of protected link
      mutually_visible = .false.
      distance = -1
      bearing = X'7fc0000'	!NaN
      type_of_conflict = ' '

c  First case: protected link (link1) causes interference to the link
c  (link2) that is displaced in frequency.  The link2 receive site must
c  be in the beam of the link2 transmitter, and the link1 transmitter
c  must be viewable from the link2 receiver.

      if (distance_1t2r .lt. far) then
         clw_lim_1t = directions(FORWARD, link1) + allow
         ccw_lim_1t = directions(FORWARD, link1) - allow
         test = bearing_1t2r
         if (within_view(ccw_lim_1t, test, clw_lim_1t)) then
            clw_lim = directions(REVERSE, link2) + allow
            ccw_lim = directions(REVERSE, link2) - allow
            test = rev_bear(bearing_1t2r)
            if (within_view(ccw_lim, test, clw_lim)) then
               bearing = bearing_1t2r
               distance = distance_1t2r
               path_len(ACROSS) = distance
               path_len(OPPOSITE) = path_length(link1)	!length of protected link
               relative_bearing(FORWARD) =
     &            clockwise_diff(bearing, directions(FORWARD, link1))
               relative_bearing(REVERSE) =
     &            clockwise_diff(rev_bear(bearing), directions(REVERSE, link2))
               db_advantage = d2u(relative_bearing, path_len, new_band)
               if (db_advantage .lt. REQUIRED_MARGIN) then
                  mutually_visible =.true.
                  type_of_conflict = 'TR'
                  return
               end if
            end if
         end if
      end if

c  Second case: protected link (link1) receives interference from the link
c  (link2) that is displaced in frequency.  The link1 receive site must be
c  in the beam of the link2 transmitter, and the latter must be viewable
c  from the link1 receiver.

      if (distance_1r2t .lt. far) then
         clw_lim_2t = directions(FORWARD, link2) + allow
         ccw_lim_2t = directions(FORWARD, link2) - allow
         test = rev_bear(bearing_1r2t)
         if (within_view(ccw_lim_2t, test, clw_lim_2t)) then
            clw_lim = directions(REVERSE, link1) + allow
            ccw_lim = directions(REVERSE, link1) - allow
            test = bearing_1r2t
            if (within_view(ccw_lim, test, clw_lim)) then
               bearing = bearing_1r2t
               distance = distance_1r2t
               path_len(ACROSS) = distance
               path_len(OPPOSITE) = path_length(link2)	!length of interfering link
               relative_bearing(FORWARD) =
     &            clockwise_diff(rev_bear(bearing), directions(FORWARD, link2))
               relative_bearing(REVERSE) =
     &            clockwise_diff(bearing, directions(REVERSE, link1))
               db_advantage = d2u(relative_bearing, path_len, new_band)
               if (db_advantage .lt. REQUIRED_MARGIN) then
                  mutually_visible =.true.
                  type_of_conflict = 'RT'
                  return
               end if
            end if
         end if
      end if

      return
      end

      function d2u(relative_bearing, path_length, band)
      real relative_bearing(2), path_length(2)
      integer band, TWELVE_MHZ, EIGHTEEN_MHZ
      integer ACROSS, OPPOSITE, FORWARD, REVERSE
      parameter(
     &   TWELVE_MHZ = 1, EIGHTEEN_MHZ = 2,
     &   ACROSS = 1, OPPOSITE = 2,
     &   FORWARD = 1, REVERSE = 2)
      distance_ratio = path_length(ACROSS)/path_length(OPPOSITE)
      d2u = 
     &   - gain(band, relative_bearing(FORWARD))
     &   - gain(band, relative_bearing(REVERSE))
     &   + 20*log10(distance_ratio)
      end
